/* $Workfile: ThorAccess.c $       $Revision: 3 $      $Modtime: 10/28/05 3:23p $  */
/*******************************************************************************

  Odin TeleSystems Inc.

  OTX Telecom Adapter Driver

  @doc PublicDemo

  @module

    A66HwBasicAnswer.c

This is a first pass at writing the entire monitoring application is C++.

The demo handles 6 FXO devices. Each device is waiting for an incoming call.


  Items to ADD:

  1. Must have a way of gracefully shutting down all threads.

  2. Must have a way of telling one thread to "Not Answer".
  
  For 1 + 2 above, How about a timer that runs every 5 seconds and looks at a .txt file 
  for configuration information. 

  Must add code for SQL access


*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>

#include <windows.h>
#include <time.h>



#include "A66HwBasic.h"
#include "OtxA66CallControl.h"
#include "OsaThrd.h"



/*******************************************************************************
 Function Prototypes
*******************************************************************************/
void demoEventFxoAnswerCaller(void *hwBase, void *hwEvent, void *hwParms, void *potsLiNum);

DWORD WINAPI demoLiThreadFunc( LPVOID lpParam );

int WaitForModelNumber(int nBd, int nLi, char *pszModelBuffer); 

extern void IPD800_Dispatch(int nBd, int nLi, char *pszVersionNum, char *pszCallType, char *pszSerialNum);
extern void JOYS_Dispatch(int nBd, int nLi, char *pszVersionNum, char *pszCallType, char *pszSerialNum);

int SendContactPhone(int nBd, int nLi, char *pszCommand, char *pszPhone ); 
int WaitForCallDetails(int nBd, int nLi, char *pszCallDetailBuffer);
int WaitForStatus(int nBd, int nLi, char *pszStatusBuffer);

int SendTones(int nBd, int nLi, char *pszTones );
int WaitForTones(int nBd, int nLi, char *pszModelBuffer, int knum);

int VerifyCheckSum(char *pszTones); 
void CreateCheckSum(char *pszTones, char *sumResult); 

int ReadContactPhones (char *pszSerialNum, char *pszPhone1, char *pszPhone2, char *pszPhone3, char *pszPhone4 );
int WriteAlarmInfo (char *pszSerialNum, char *pszType, char *pszInfo);
int WriteCallStatus (char *pszSerialNum, char *pszInfo);
void TrimPhoneNumber(char *pszOldPhone, char *pszNewPhone);

int intShuttingDown;		/* True if System is Shutting Down */
int intInCall[108];			/* True if Thread is in the middle of a Call (18 Bds * 6 Lines)*/

#define SIZE_OF_QUEUE 256 // Not sure what the size should be


/*******************************************************************************
 Global Data
*******************************************************************************/
extern OsaMutexS   g_hOtxA66MutexDtmfDigit;
extern DemoHwEvent g_nOtxA66FxoTaskCallEvent[];
extern OTX_UINT32  g_nOtxA66FxoTaskCallParms[];
extern OTX_UINT32  g_nOtxA66FxoTaskCallDigitIndex[];
extern char        *g_pOtxA66FxoTaskCallDigitIndex[];

static char        g_queue[SIZE_OF_QUEUE];
static OTX_BOOL    g_nContinuousAnswer  = OTX_FALSE; // Not initialize (not running)



/*******************************************************************************
*******************************************************************************/
int main(int argc, char *argv[])
{

  DWORD dwThreadId[6], dwThrdParam[6]; 
  HANDLE hThread[6]; 
  int i;
  int nThreadInit[6];
  int bRunAnswer;
  struct DemoEnvT OtxCallControlSetup;

  OtxCallControlSetup.bContinueOnError = 0;
  OtxCallControlSetup.bExitOnError = 0;
  OtxCallControlSetup.boardNo = 0;
  OtxCallControlSetup.fxoStartNo = 0;
  OtxCallControlSetup.fxoMaxNo = 6;
  OtxCallControlSetup.fxsMaxNo = 0;
  OtxCallControlSetup.law = OTX_DATA_FORMAT_COMPAND8_U_LAW;
  OtxCallControlSetup.logFileName = 0;
//  OtxCallControlSetup.nBoardType = OTX_DEVICE_ARNI_6_FXO_PCI_PLUS;
  OtxCallControlSetup.nBoardType = OTX_DEVICE_ARNI_6X6_PCI;
  OtxCallControlSetup.nSetId = OTX_CST_POTS_CFG_SET_USA;
  OtxCallControlSetup.outFileName = 0;

  OtxA66BoardInit(&OtxCallControlSetup, 
                  NULL); // Log utility callback not available at this time

  // Check if the board initialized correctly
  if (!OtxA66BoardIsValid()) {
    printf("Error: Board did not initialize correctly\n");
    exit(1);
  }

  // Check if all 6 line interfaces initialized correctly
  for (i = 0; i < 6; i++) {
    if (!OtxA66FxoIsValid(OtxCallControlSetup.boardNo, i)) {
      printf("Error: Li[%d] did not initialize correctly\n", i);
      exit(1);
    }
    dwThrdParam[i] = i; // Pass 0, 1, 2, ..., 5 as the thread parameter (the Li number)
  }



  // Create the 6 line interface threads that will run individually
  for (i = 0; i < 6; i++) {
    nThreadInit[i] = i;
    hThread[i] = CreateThread( 
      NULL,                        // no security attributes 
      0,                           // use default stack size  
      demoLiThreadFunc,            // thread function 
      &nThreadInit[i],             // argument to thread function 
      0,                           // use default creation flags 
      &dwThreadId[i]);             // returns the thread identifier 

    // Check the return value for success. 
    if (hThread[i] == NULL) {
      printf("Error: CreateThread[%d] failed\n", i); 
      exit(1);
    }

    Sleep(400); // Let each task start.
  }

  g_nContinuousAnswer  = OTX_TRUE;
  bRunAnswer = OTX_TRUE;

  while(bRunAnswer) {
    Sleep(200); 
  }

  g_nContinuousAnswer  = OTX_FALSE;

  printf("Closing ARNI board number %d\n", OtxCallControlSetup.boardNo); 

  OtxA66BoardOnClose();

  return 0; 
} 

/*******************************************************************************

                              Hardware Event Callback

Collects the hardware/software events produced by the OTX driver.
The events are defined in DemoHwEvent enum.

*******************************************************************************/
void demoEventFxoAnswerCaller(
  void *hwBase, 
  void *hwEvent, 
  void *hwParms,
  void *potsLiNum
)
{
  OTX_INT32  *pTmp1;
  OTX_UINT32 *pTmp2;
  OTX_UINT32 *pTmpLiNum;
  OTX_UINT32  *pDigitIndex;
  char *pDigit;
  char *pDigitBuffer = 0;
  DemoHwEvent *pTmpEvent;

  pTmp1  = (OTX_UINT32 *) hwBase; // Currently used to hold the board number
  pTmp2  = (OTX_UINT32 *) hwParms;
  pTmpLiNum  = (OTX_UINT32 *) potsLiNum;
  pTmpEvent = (DemoHwEvent *) hwEvent;

  // The following should be a queue but for this app a single
  // location will work.
  if(g_nContinuousAnswer) {
    // For now all we want is the event data
    if (g_nOtxA66FxoTaskCallEvent[*pTmpLiNum] == HwEvent_unused) {
        g_nOtxA66FxoTaskCallEvent[*pTmpLiNum] = *pTmpEvent;
        g_nOtxA66FxoTaskCallParms[*pTmpLiNum] = *pTmp2;
      }

    // Collect the DTMF Digits
    if(*pTmpEvent == HwEvent_digit_avail) {
      pDigit  = (char *) hwParms;
      // Make sure only one thread can accesses this data
      OsaMutexLock(&g_hOtxA66MutexDtmfDigit);
      pDigitIndex = (OTX_UINT32  *) &g_nOtxA66FxoTaskCallDigitIndex[*pTmpLiNum]; 

      pDigitBuffer = g_pOtxA66FxoTaskCallDigitIndex[*pTmpLiNum];
      if(pDigitBuffer != 0) {
        *(pDigitBuffer + *pDigitIndex) = *pDigit;
        *pDigitIndex = *pDigitIndex + 1;
        if(*pDigitIndex >= OTX_A66_CC_MAX_DTMF_DIGITS) {
          // Digit buffer is full. What to do.
          *pDigitIndex = 0; // Start over
        }
      }
      OsaMutexUnlock(&g_hOtxA66MutexDtmfDigit);
    }
  }  
  
  return;
}



/*******************************************************************************

                          Individual Inbound Tread Processor


Wait for a Call
When a Call comes in, Answer it immediately
Wait a while so that the other end is ready to receive the DTMF digits that we are about to send

A max of 4 attempts in either direction before hang-up. Reset the Counter before the 
1st transmission of new information in either direction.

Model, Version, Call Type and Serial Number

Bytes 	0-1 Model #
		2-3 Version #
		4-5 Call Type
		6-14 Serial Number
		15-16 CheckSum

If Call Type = 11 then call subroutine to send contact info and receive alarm info
If Call Type = 22 then call subroutine to receive contact calling details
If Call Type = 33 then call subroutine to receive status details

*******************************************************************************/
DWORD WINAPI demoLiThreadFunc( LPVOID lpParam ) 
{ 
  char pszCallNumber[64 + 1]; // End with a null
//    char pszDtmfBuffer[128];
  char pszModelBuffer[17 + 1];
  char pszModelNumber[2 + 1];
  char pszVersionBuffer[2 + 1];
  char pszVersionNumber[2 + 1];
  char pszCallType[2 + 1];
  char pszSerialNumber[9 + 1];

  int *pLi = (int*)lpParam;

  int nBd, nLi;
  int kCounter = 0;

  nBd = OtxA66BoardGetNumber(*pLi);	// Get the Board Number for this Thread
  nLi = OtxA66FxoGetLiNumber(nBd, *pLi); // Get the Line Number on this Board for this Thread

  // Setup event handler callback
  OtxA66BoardInitEventHandler(nLi,
                              (void *) &nBd, 
                              demoEventFxoAnswerCaller);
  Sleep(200);


  printf("Board [%d] Line [%d] - Initialized\n", nBd, nLi); 

  // Our event queue only holds one event.
  g_nOtxA66FxoTaskCallEvent[nLi] = HwEvent_unused;

  while (1) {
    // Start each time with clean buffers
    memset(pszCallNumber, '\0', 64 + 1);
    memset(pszModelBuffer, '\0', 17 + 1);
    memset(pszModelNumber, '\0', 2 + 1);
    memset(pszVersionBuffer, '\0', 2 + 1);
    memset(pszVersionNumber, '\0', 2 + 1);
    memset(pszCallType, '\0', 2 + 1);
    memset(pszSerialNumber, '\0', 9 + 1);

// Clear "In Call" Flag for this Line
	intInCall[(nBd * 6) + nLi] = 0;

    OtxA66FxoWaitForCall(nBd, nLi, pszCallNumber, 64);    

    printf("Board [%d] Line [%d] - Incoming Call Received from \"%s\" \n", nBd, nLi, pszCallNumber);

// Answer, 
    OtxA66FxoGoOffhook(nBd, nLi);

// Set "In Call" Flag for this Line
	intInCall[(nBd * 6) + nLi] = 1;

// Wait a while so that the other end is ready to receive the DTMF digit that we are about to send him
    Sleep(200);

// Should be off hook, maybe far end hung up. Check It.
    if(OtxA66FxoIsOffhook(nBd, nLi)) {
    
// Send DTMF Digits Requesting the Model, Version, Call Type and Serial Number
// Wait for the Model, Version, Call Type and Serial Number

		if (WaitForModelNumber(nBd, nLi, pszModelBuffer) == 0) {

// Get Model Number from the ModelBuffer
			strncpy_s(pszModelNumber, sizeof(pszModelNumber),pszModelBuffer, 2); 

// Get Version Number from the ModelBuffer
			strncpy_s(pszVersionNumber,  sizeof(pszVersionNumber),pszModelBuffer + 2, 2); 

// Get Call Type from the ModelBuffer
			strncpy_s(pszCallType,  sizeof(pszCallType),pszModelBuffer + 4, 2); 

// Get Serial Number from the ModelBuffer
			strncpy_s(pszSerialNumber,  sizeof(pszSerialNumber),pszModelBuffer + 6, 9); 

			printf("Board [%d] Line [%d] - Received Model \"%s\" Version \"%s\" Call Type \"%s\" SerialNumber \"%s\" \n", nBd, nLi, pszModelNumber, pszVersionNumber, pszCallType, pszSerialNumber);


// Now call the routine specific to the Model Number

			if (strcmp(pszModelNumber, "01") == 0) {
				IPD800_Dispatch(nBd, nLi, pszVersionNumber, pszCallType, pszSerialNumber );
			}

			if (strcmp(pszModelNumber, "02") == 0) {
				JOYS_Dispatch(nBd, nLi, pszVersionNumber, pszCallType, pszSerialNumber );
			}
		
		} /* end of Get Model Number */

	} /* end of Check OffHook */


    OtxA66FxoGoOnhook(nBd, nLi);

	intInCall[(nBd * 6) + nLi] = 0;

	if (intShuttingDown == 1) break;	/* Exit While Loop */

  } // end of 'while'

  printf("Board [%d] Line [%d] - No Longer In Service\n", nBd, nLi); 

  return 0; 
} 
 




/*******************************************************************************

                Send Command for Model Number & Wait for Ack/Nak
                                 then
  Wait for Model #, Version #, Call Type and Serial Number - Send Ack or Nak

On Entry:	nBd = Board #
			nLi = Line # on Board
			Buffer to contain Tones Received

On Exit:	0 = Command Sent and Ack'ed, Tones received successful
			1 = something was not successful
			2 = OnHook Detected

*******************************************************************************/
int WaitForModelNumber(int nBd, int nLi, char *pszModelBuffer) 
{
  int kcounter;
  int kret;
  
// Send DTMF Digits Requesting the Model, Version, Call Type and Serial Number

	kcounter = 0;
kc1:
	if (kcounter < 4) {
		kret = SendTones(nBd, nLi, "0000");
		if (kret == 2) return 2;	// Remote Hang up
		if (kret == 1) {
			kcounter++;
			goto kc1;
		}
	} 
	else return 1;
  

// Wait for DTMF digits to be received

	kcounter = 0;
kc2:
	if (kcounter < 4) {
		kret = WaitForTones(nBd, nLi, pszModelBuffer, 17);
		if (kret == 2) return 2;	// Remote Hang up
		if (kret == 1) {
			kcounter++;
			goto kc2;
		}
	} 
	else return 1;

	return 0; 
} 



/*******************************************************************************

                   Send Command for Call Details & Wait for Ack/Nak
                                       then
                      Wait for Call Details - Send Ack or Nak

On Entry:	nBd = Board #
			nLi = Line # on Board
			Buffer to contain Tones Received

On Exit:	0 = Command Sent and Ack'ed, Tones received successful
			1 = something was not successful
			2 = OnHook Detected

*******************************************************************************/
int WaitForCallDetails(int nBd, int nLi, char *pszCallDetailBuffer) 
{
  int kcounter;
  int kret;
  
// Send DTMF Digits Requesting Call Details

	kcounter = 0;
kc1:
	if (kcounter < 4) {
		kret = SendTones(nBd, nLi, "2103");
		if (kret == 2) return 2;	// Remote Hang up
		if (kret == 1) {
			kcounter++;
			goto kc1;
		}
	} 
	else return 1;
  

// Wait for DTMF digits to be received

	kcounter = 0;
kc2:
	if (kcounter < 4) {
		kret = WaitForTones(nBd, nLi, pszCallDetailBuffer, 6);
		if (kret == 2) return 2;	// Remote Hang up
		if (kret == 1) {
			kcounter++;
			goto kc2;
		}
	} 
	else return 1;

  return 0; 
} 


/*******************************************************************************

      Send Command for Alarm System to send Call Status & Wait for Ack/Nak
                                  then
                  Wait for Status Info - Send Ack or Nak

On Entry:	nBd = Board #
			nLi = Line # on Board
			Buffer to contain Tones Received

On Exit:	0 = Command Sent and Ack'ed, Tones received successful
			1 = something was not successful
			2 = OnHook Detected

*******************************************************************************/
int WaitForStatus(int nBd, int nLi, char *pszStatusBuffer) 
{
  int kcounter;
  int kret;
  
// Send DTMF Digits Requesting Status Information

	kcounter = 0;
kc1:
	if (kcounter < 4) {
		kret = SendTones(nBd, nLi, "2406");
		if (kret == 2) return 2;	// Remote Hang up
		if (kret == 1) {
			kcounter++;
			goto kc1;
		}
	} 
	else return 1;


// Wait for DTMF digits to be received

	kcounter = 0;
kc2:
	if (kcounter < 4) {
		kret = WaitForTones(nBd, nLi, pszStatusBuffer, 4);
		if (kret == 2) return 2;	// Remote Hang up
		if (kret == 1) {
			kcounter++;
			goto kc2;
		}
	} 
	else return 1;

  return 0; 
} 






/*******************************************************************************

        Send Command to Wait for Contact Phone Number & Wait for Ack/Nak
                                  then
                Send Contact Phone Number & Wait for Ack/Nak

On Entry:	nBd = Board #
			nLi = Line # on Board
			Command Tones to Send (for which Contact Telephone Number)
			Contact Telephone Number Tones - we will calculate CheckSum here

On Exit:	0 = Command Sent and Ack'ed, Tones received successful
			1 = something was not successful
			2 = OnHook Detected

If we receive 4 Nak's when sending either the Command or the Contact Phone #,
we will return a '1' indication.

*******************************************************************************/
int SendContactPhone(int nBd, int nLi, char *pszCommand, char *pszPhone ) 
{
	char pszCheckSum[4+1];
	char pszNewPhone[64+1];

	int kcounter;
	int kret;

// Send DTMF Digits Telling Alarm System to Expect Contact Phone Number

	kcounter = 0;
kc1:
	if (kcounter < 4) {
		kret = SendTones(nBd, nLi, pszCommand);
		if (kret == 2) return 2;	// Remote Hang up
		if (kret == 1) {
			kcounter++;
			goto kc1;
		}
	} 
	else return 1;


// Calculate the Checksum and append to the Contact Phone Number

	CreateCheckSum(pszPhone, pszCheckSum);
	strcpy_s (pszNewPhone, sizeof(pszNewPhone),pszPhone);
	strcat_s (pszNewPhone,  sizeof(pszNewPhone),pszCheckSum);


// Send DTMF Digits Containing the Contact Phone Number + CheckSum

	kcounter = 0;
kc2:
	if (kcounter < 4) {
		kret = SendTones(nBd, nLi, pszNewPhone);
		if (kret == 2) return 2;	// Remote Hang up
		if (kret == 1) {
			kcounter++;
			goto kc2;
		}
	} 
	else return 1;

  return 0; 
} 
 






/*******************************************************************************

                Send Tones to the Alarm System - Wait for Ack or Nak

On Entry:	nBd = Board #
			nLi = Line # on Board
			Tones to send - CheckSum must already be attached

On Exit:	0 = send and confirmed successful
			1 = not sent or not successful
			2 = OnHook Detected

*******************************************************************************/
int SendTones(int nBd, int nLi, char *pszTones ) 
{
	char pszAckNak[4+1]; // Need room for NULL
  
	memset(pszAckNak, '\0', 5);

// Send DTMF Digits - as passed by caller
	OtxA66FxoSendDtmf(nBd, nLi, pszTones);

	Sleep(200); // delay between

// Wait for Ack or Nak DTMF digits to be received
	OtxA66FxoWaitForDtmf(nBd, nLi, 5000, pszAckNak, 4);

// Check to see if the caller hung up
	if(!OtxA66FxoIsOffhook(nBd, nLi)) return 2;

// Now check for the correct number of DTMF digits
	if (strlen(pszAckNak) != 4) return 1;


//  Now check for Nak tones received
	if (strcmp(pszAckNak, "9818") == 0) return 1;


//  Now check for Ack tones received
  if (strcmp(pszAckNak, "9009") == 0) return 0;

  
  return 1; 
} 
 



/*******************************************************************************
                           
                       Wait for Tones - Send Ack or Nak

On Entry:	nBd = Board #
			nLi = Line # on Board
			Buffer to contain Tones Received
			Number of Tones Expected

On Exit:	0 = received successful and Ack Sent
			1 = not received successful and Nak Sent
			2 = OnHook Detected

*******************************************************************************/
int WaitForTones(int nBd, int nLi, char *pszModelBuffer, int knum) 
{


// Wait for DTMF digits to be received
    /*
    ** knum contains up to 4 DTMF digits;
    */
    OtxA66FxoWaitForDtmf(nBd, nLi, 5000, pszModelBuffer, knum);

// Check to see if the caller hung up
	if(!OtxA66FxoIsOffhook(nBd, nLi)) return 2;

// Did we receive the number of tones we were expecting?  - If NO, send Nak
	if (strlen(pszModelBuffer) == knum) {
		OtxA66FxoSendDtmf(nBd, nLi, "9918");

		// Check to see if the caller hung up
		if(!OtxA66FxoIsOffhook(nBd, nLi)) return 2;
		return 1;
    }

// Now check the CheckSum of the tones received  - If not valid, send Nak

    if (VerifyCheckSum(pszModelBuffer) == 1) {
		OtxA66FxoSendDtmf(nBd, nLi, "9918");

		// Check to see if the caller hung up
		if(!OtxA66FxoIsOffhook(nBd, nLi)) return 2;
		return 1;
    }

// Everything looks good - send Ack
	OtxA66FxoSendDtmf(nBd, nLi, "9009");

// Check to see if the caller hung up
	if(!OtxA66FxoIsOffhook(nBd, nLi)) return 2;

	return 0; 
} 




/*******************************************************************************
                CheckSum Verification Routine - for Received Tones

The last 2 tones in the received string are the CheckSum
Now Add the value of all tones in the string (except the last 2 tones).
The * and # tones (if any) will add as 9 each.
Convert the total to a string.  

Take the last 2 digits of this string and compare to the last 2 digits of the Received Tones
If they match, then the tones are valid - send tones "9009"
If not, reject the transmission - send tones "9918"


Example1:

  If you Receive  01 01 11 000 001 001 06  -  The last 2 digits, 06, is the CheckSum

  Calculate the CheckSum 0+1 + 0+1 + 1+1 + 0+0+0 + 0+0+1 + 0+0+1  = 6 -> change to "06" for 2 digits

  06 Received Matches 06 Calculated, so the reception of Tones is OK


Example2:

  If you Receive  01 09 11 001 009 001 23  -  The last 2 digits, 23, is the CheckSum

  Calculate the CheckSum 0+1 + 0+9 + 1+1 + 0+0+1 + 0+0+9 + 0+0+1  = 23 

  23 Received Matches 23 Calculated, so the reception of Tones is OK


*******************************************************************************/
int VerifyCheckSum(char *pszTones) 
{
  
   int kj;
   int ksum;
   int strLen;
   char kchar;
   char cksm[3];
   char temp[3];

   memset(cksm, '\0', 3);
   memset(temp, '\0', 3);

   kj = strlen(pszTones);
   cksm[0] = pszTones[kj-3];
   cksm[1] = pszTones[kj-2];

   ksum = 0;
   strLen = (int) strlen(pszTones) - 2;
   for (kj = 0; kj < strLen; kj++) {
      kchar = g_queue[kj];
      if ((kchar == '*') || (kchar == '#')) {
         ksum = ksum + 9;
      } else {
         if ((kchar >= '0') && (kchar <= '9')) {
            ksum = ksum + atoi(&kchar);
         }
      }
   }

   // convert ksum to a numeric string - if less than 2 chars, make it 2 chars with a leading 0

   itoa(ksum, temp, 3);
   if (strlen(temp) < 2) {
     temp[1] = temp[0];
     temp[0] = '0';
   }

   // Now compare the last 2 chars of psxTones with the string above
   // If Match, return 0, otherwise return 1 

   if (strcmp(cksm,temp) == 0) {
     return 0;
   } else {
       return 1;
   }
}



/*******************************************************************************

                  Create CheckSum Routine - for Tones to be Sent

*******************************************************************************/
void CreateCheckSum(char *pszTones, char *sumResult) 
{
  
   int kj;
   int ksum;
   int strLen;
   char kchar;
   char kcharBuf[2];
   char cksm[3];
   char *temp;

   // Pass result back to caller
   temp= sumResult;

   memset(cksm, '\0', 3);
   memset(temp, '\0', 3);

   kj = strlen(pszTones);
   cksm[0] = pszTones[kj-3];
   cksm[1] = pszTones[kj-2];

   ksum = 0;
   strLen = (int) strlen(pszTones);
   for (kj = 0; kj < strLen; kj++) {
      kchar = g_queue[kj];
      if ((kchar == '*') || (kchar == '#')) {
         ksum = ksum + 9;
      } else {
         if ((kchar >= '0') && (kchar <= '9')) {
            kcharBuf[0] = kchar;
            kcharBuf[1] = (char) '/0';
            ksum = ksum + atoi(kcharBuf);   // add numeric value of kchar here
         }
      }
   }

   // convert ksum to a numeric string - if less than 2 chars, make it 2 chars with a leading 0

   itoa(ksum, temp, 3);
   if (strlen(temp) < 2) {
     temp[1] = temp[0];
     temp[0] = '0';
   }

   return;
}



 
/*******************************************************************************

          Read Customer Contact Record and Return Telephone Numbers

*******************************************************************************/
int ReadContactPhones (char *pszSerialNum, char *pszPhone1, char *pszPhone2, char *pszPhone3, char *pszPhone4 )
{
	int numread;
	char pszFileName[160];
	char pszSerialDir[6];
	char pszTempPhone[20];
	char pszCrLf[2];

	FILE *stream;
	errno_t err;

	strncpy_s (pszSerialDir, sizeof(pszSerialDir), pszSerialNum, 6);

	sprintf_s (pszFileName, sizeof(pszFileName), "C:\\AlarmData\\Contacts\\%s\\%s.txt", pszSerialDir, pszSerialNum);
 
// Open for read (will fail if file does not exist)

    if( (err  = fopen_s( &stream, pszFileName, "r" )) !=0 ) return (1);

	numread = fread( pszTempPhone, sizeof( char ), 25, stream );
	TrimPhoneNumber (pszTempPhone, pszPhone1);
	numread = fread( pszCrLf, sizeof( char ), 3, stream );

	numread = fread( pszTempPhone, sizeof( char ), 25, stream );
	TrimPhoneNumber (pszTempPhone, pszPhone2);
	numread = fread( pszCrLf, sizeof( char ), 3, stream );

	numread = fread( pszTempPhone, sizeof( char ), 25, stream );
	TrimPhoneNumber (pszTempPhone, pszPhone3);
	numread = fread( pszCrLf, sizeof( char ), 3, stream );

	numread = fread( pszTempPhone, sizeof( char ), 25, stream );
	TrimPhoneNumber (pszTempPhone, pszPhone4);
	numread = fread( pszCrLf, sizeof( char ), 3, stream );

// Close stream if it is not NULL 
    if( stream) {
		if ( fclose( stream ) ) return (0);
	}
	return (0);

}



/*******************************************************************************

          Write Alarm Information or Call Detail Information to File

pszSerialNum	= 9 Digit Serial Number of Alarm System
pszType			= A for Alarm,  or  D for Details
pszInfo			= Alarm String  or  Detail String

*******************************************************************************/
int WriteAlarmInfo (char *pszSerialNum, char *pszType, char *pszInfo)
{
	int numread;
	char pszFileName[160];
	char pszSerialDir[7];
	char pszTemp[45];
	char pszSpace[21] = "                    ";
	char pszCrLf[4] = "|\r\n";
	char pszDate[9];
	char pszNewDate[11];
	char pszTime[9];

	FILE *stream;
	errno_t err;

	strncpy_s (pszSerialDir, sizeof(pszSerialDir), pszSerialNum, 6);

	sprintf_s (pszFileName, sizeof(pszFileName), "C:\\AlarmData\\Alarms\\%s\\%s.txt", pszSerialDir, pszSerialNum);

	_strdate_s (pszDate, sizeof(pszDate));
	_strtime_s (pszTime, sizeof(pszTime));

	pszTemp[0] = pszDate[0];
	pszTemp[1] = pszDate[1];
	pszTemp[2] = '-';
	pszTemp[3] = pszDate[3];
	pszTemp[4] = pszDate[4];
	pszTemp[5] = '-';
	pszTemp[6] = '2';
	pszTemp[7] = '0';
	pszTemp[8] = pszDate[6];
	pszTemp[9] = pszDate[7];
	pszTemp[10] = ' ';
	pszTemp[11] = '\0';

	strcat_s (pszTemp, sizeof(pszTemp), pszTime);
	strncat_s (pszTemp, sizeof(pszTemp), pszSpace, 1);

	strcat_s (pszTemp, sizeof(pszTemp), pszInfo);

	strncat_s (pszTemp, sizeof(pszTemp), pszSpace, (sizeof(pszTemp) - 3) - (strlen(pszInfo) + 20));

	strcat_s (pszTemp, sizeof(pszTemp), pszCrLf);

// Open for append - will fail if file does not exist - then open for write

	if( (err  = fopen_s( &stream, pszFileName, "a+" )) !=0 ) {
		if( (err  = fopen_s( &stream, pszFileName, "w" )) !=0 ) return(1);
	}

	numread = fwrite( pszTemp, sizeof( char ), 20, stream );

// Close stream if it is not NULL 
    if( stream) {
		if ( fclose( stream ) ) return (0);
	}
	return (0);

}



/*******************************************************************************

                    Write Call Status Information to File

This routine will open Status File, locate the record for this Alarm System, 
Write the Date, Time, and Status Info to that record.

This routine will then append the information to the end of a File whose name is
the Serial Number.

pszSerialNum	= 9 Digit Serial Number of Alarm System
pszStatus		= Status Information from Call

*******************************************************************************/
int WriteCallStatus (char *pszSerialNum, char *pszInfo)
{
	int numread;
	char pszFileName[160];
	char pszSerialDir[7];
	char pszTemp[45];
	char pszSpace[21] = "                    ";
	char pszCrLf[4] = "|\r\n";
	char pszDate[9];
	char pszNewDate[11];
	char pszTime[9];

	FILE *stream;
	errno_t err;

	strncpy_s (pszSerialDir, sizeof(pszSerialDir), pszSerialNum, 6);

	sprintf_s (pszFileName, sizeof(pszFileName), "C:\\AlarmData\\Status\\%s\\%s.txt", pszSerialDir, pszSerialNum);

	_strdate_s (pszDate, sizeof(pszDate));
	_strtime_s (pszTime, sizeof(pszTime));

	pszTemp[0] = pszDate[0];
	pszTemp[1] = pszDate[1];
	pszTemp[2] = '-';
	pszTemp[3] = pszDate[3];
	pszTemp[4] = pszDate[4];
	pszTemp[5] = '-';
	pszTemp[6] = '2';
	pszTemp[7] = '0';
	pszTemp[8] = pszDate[6];
	pszTemp[9] = pszDate[7];
	pszTemp[10] = ' ';
	pszTemp[11] = '\0';

	strcat_s (pszTemp, sizeof(pszTemp), pszTime);
	strncat_s (pszTemp, sizeof(pszTemp), pszSpace, 1);

	strcat_s (pszTemp, sizeof(pszTemp), pszInfo);

	strncat_s (pszTemp, sizeof(pszTemp), pszSpace, (sizeof(pszTemp) - 3) - (strlen(pszInfo) + 20));

	strcat_s (pszTemp, sizeof(pszTemp), pszCrLf);

// Open for append (will fail if file does not exist)

	if( (err  = fopen_s( &stream, pszFileName, "a+" )) !=0 ) {
		if( (err  = fopen_s( &stream, pszFileName, "w" )) !=0 ) return(1);
	}

	numread = fwrite( pszTemp, sizeof( char ), 20, stream );

// Close stream if it is not NULL 
    if( stream) {
		if ( fclose( stream ) ) return (0);
	}
	return (0);

}


/*******************************************************************************

                  Trim and Remove "-" from Phone Number

*******************************************************************************/
void TrimPhoneNumber(char *pszOldPhone, char *pszNewPhone) 
{
    unsigned int kj;
	unsigned int kx;
    char kchar;

	kx = 0;
	pszNewPhone[kx] = '\0';
	for (kj = 0; kj < strlen(pszOldPhone); kj++) {
		kchar = pszOldPhone[kj];
		if (kchar >= '0' && kchar <= '9') {
            pszNewPhone[kx] = kchar;
			pszNewPhone[kx + 1] = '\0';
			kx++;
		}
	}
	return;
}
