/* $Workfile: JOY-S.c $       $Revision: 3 $      $Modtime: 10/28/05 3:23p $  */
/*******************************************************************************

	                  Interface to JOY-S Alarm System

Only routines that are differnt from the standard routines will be located here

*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>

#include <windows.h>



#include "A66HwBasic.h"
#include "OtxA66CallControl.h"


void JOYS_Dispatch(int nBd, int nLi, char *pszVersionNum, char *pszCallType, char *pszSerialNum);

int JOYS_01_AlarmReporting(int nBd, int nLi, char *pszSerialNum ); 
int JOYS_01_CallDetails(int nBd, int nLi, char *pszSerialNum ); 
int JOYS_01_StatusCall(int nBd, int nLi, char *pszSerialNum ); 
int JOYS_01_WaitForAlarmInfo(int nBd, int nLi, char *pszAlarmBuffer); 

extern int SendContactPhone(int nBd, int nLi, char *pszCommand, char *pszPhone ); 
extern int WaitForCallDetails(int nBd, int nLi, char *pszCallDetailBuffer);
extern int WaitForStatus(int nBd, int nLi, char *pszStatusBuffer);

extern int SendTones(int nBd, int nLi, char *pszTones );
extern int WaitForTones(int nBd, int nLi, char *pszModelBuffer, int knum);

extern int VerifyCheckSum(char *pszTones); 
extern void CreateCheckSum(char *pszTones, char *sumResult); 

extern int ReadContactPhones (char *pszSerialNum, char *pszPhone1, char *pszPhone2, char *pszPhone3, char *pszPhone4 );
extern int WriteAlarmInfo (char *pszSerialNum, char *pszType, char *pszInfo);
extern int WriteCallStatus (char *pszSerialNum, char *pszInfo);

/*******************************************************************************

                     Dispatch Routine for Model JOYS

*******************************************************************************/
void JOYS_Dispatch(
	int nBd,
	int nLi,
	char *pszVersionNum, 
	char *pszCallType,
	char *pszSerialNum ) 
{

// Only One of the 3 Call Types was specified in the Model Buffer.
// No need to check for hang up after any of them, as we will hang up anyway.

// If Call Type = "11" = Alarm Call

	if (strcmp(pszCallType, "11") == 0) {
		if (strcmp(pszVersionNum, "01") == 0) {
			JOYS_01_AlarmReporting( nBd, nLi, pszSerialNum );
		}
	}


// If Call Type = "22" = Call Details

	if (strcmp(pszCallType, "22") == 0) {
		if (strcmp(pszVersionNum, "01") == 0) {
			JOYS_01_CallDetails( nBd, nLi, pszSerialNum );
		}
	}


// If Call Type = "33" = Status Call

	if (strcmp(pszCallType, "33") == 0) {
		if (strcmp(pszVersionNum, "01") == 0) {
			JOYS_01_StatusCall( nBd, nLi, pszSerialNum );
		}
	}

	return;
}




/*******************************************************************************

                     Call Type = "11" - Alarm Reporting Call

On Entry:	nBd = Board #
			nLi = Line # on Board
			Serial Number

On Exit:	0 = Everything successful and Ack Sent
			1 = One or more transmissions NOT successful and Nak Sent
			2 = OnHook Detected

Get the "Contacts" from the File
Send Contact Phone Numbers to the Alarm System
Get the Alarm Info from the Alarm System

*******************************************************************************/
int JOYS_01_AlarmReporting(int nBd, int nLi, char *pszSerialNum ) 
{
    char pszPhone1[20+1];
    char pszPhone2[20+1];
    char pszPhone3[20+1];
    char pszPhone4[20+1];
	char pszAlarmBuffer[255];
  
	int kret;

	memset(pszPhone1, '\0',  21);
	memset(pszPhone2, '\0',  21);
	memset(pszPhone3, '\0',  21);
	memset(pszPhone4, '\0',  21);
	memset(pszAlarmBuffer,  '\0', 255);

// Get Contact Phone Numbers from File

	ReadContactPhones (pszSerialNum, pszPhone1, pszPhone2, pszPhone3, pszPhone4);


// Send Command and 1st Contact Phone Number to Alarm System

	kret = SendContactPhone(nBd, nLi, "1102", pszPhone1);
	if (kret == 2) return 2;
	if (kret == 1) return 1;


// Send Command and 2nd Contact Phone Number to Alarm System

	kret = SendContactPhone(nBd, nLi, "1203", pszPhone2);
	if (kret == 2) return 2;
	if (kret == 1) return 1;
    

// Send Command and 3rd Contact Phone Number to Alarm System

	kret = SendContactPhone(nBd, nLi, "1304", pszPhone3);
	if (kret == 2) return 2;
	if (kret == 1) return 1;


// Send Command and 4th Contact Phone Number to Alarm System

	kret = SendContactPhone(nBd, nLi, "1405", pszPhone4);
	if (kret == 2) return 2;
	if (kret == 1) return 1;


// Send Command and Wait for Alarm Information to be Sent from Alarm System

	kret = JOYS_01_WaitForAlarmInfo(nBd, nLi, pszAlarmBuffer);
	if (kret == 2) return 2;
	if (kret == 1) return 1;


// Write Alarm Info to "Alarms" File

	WriteAlarmInfo (pszSerialNum, "A", pszAlarmBuffer);


	return 0; 
} 




/*******************************************************************************

                     Call Type = "22" - Call Details Call

On Entry:	nBd = Board #
			nLi = Line # on Board
			Serial Number

On Exit:	0 = Call Details received successful and Ack Sent
			1 = not received successful and Nak Sent
			2 = OnHook Detected

Get the Call Details from the Alarm System
Save the Call Details to the Alarms SQL DB

*******************************************************************************/
int JOYS_01_CallDetails(int nBd, int nLi, char *pszSerialNum ) 
{
	char pszCallDetailBuffer[6+1]; // End with a null

	int kret;

	memset(pszCallDetailBuffer,  '\0', 7);

// Send Command and Wait for Call Details

	kret = WaitForCallDetails(nBd, nLi, pszCallDetailBuffer);
	if (kret == 2) return 2;
	if (kret == 1) return 1;



// Write Call Details Info to "Alarms" File

	WriteAlarmInfo (pszSerialNum, "D", pszCallDetailBuffer);


  return 0; 
} 





/*******************************************************************************

                     Call Type = "33" - Status Call

On Entry:	nBd = Board #
			nLi = Line # on Board
			Serial Number

On Exit:	0 = Status received successful and Ack Sent
			1 = not received successful and Nak Sent
			2 = OnHook Detected

Get the Status from the Alarm System
Save the Status to the Alarms File

*******************************************************************************/
int JOYS_01_StatusCall(int nBd, int nLi, char *pszSerialNum ) 
{
    char pszStatusBuffer[4+1];

	int kret;
    
	memset(pszStatusBuffer, '\0', 5);


// Send Command and Wait for Alarm Status

	kret = WaitForStatus( nBd, nLi, pszStatusBuffer);
	if (kret == 2) return 2;
	if (kret == 1) return 1;


// Write Status Info to Files

	WriteCallStatus (pszSerialNum, pszStatusBuffer);


  return(0);
} 




/*******************************************************************************

    Send Command to Alarm System to Send Alarm Information & Wait for Ack/Nak
                                  then
               Wait for Alarm Information - Send Ack or Nak

On Entry:	nBd = Board #
			nLi = Line # on Board
			Buffer to contain Tones Received - must be at least 255 characters

On Exit:	0 = Command Sent and Ack'ed, Tones received successful
			1 = something was not successful
			2 = OnHook Detected

Send Command then Wait for Alarms
Check Alarms to see if any Alarm Information or Empty Information
If not Empty, Wait for More Alarms
Repeat until Empty Information received

*******************************************************************************/
int JOYS_01_WaitForAlarmInfo(int nBd, int nLi, char *pszAlarmBuffer) 
{
  char pszTempBuffer[21];
  int kcounter;
  int kret;
  
  
// Send DTMF Digits Requesting Alarm Information

	kcounter = 0;
kc1:
	if (kcounter < 4) {
		kret= SendTones(nBd, nLi, "2002");
		if (kret == 2) return 2;	// Remote Hang up
		if (kret == 1) {
			kcounter++;
			goto kc1;
		}
	}
	else return 1;


// Wait for DTMF digits to be received

	kcounter = 0;
kc2:
	if (kcounter < 4) {
		kret = WaitForTones(nBd, nLi, pszTempBuffer, 21);
		if (kret == 2) return 2;	// Remote Hang up
		if (kret == 1) {
			kcounter++;
			goto kc2;
		}
	}
	else return 1;


// Check if the Tones Received (not including the CheckSum) indicates Empty Alarm Info

	if (strncmp(pszTempBuffer, "9000000000000000009", 19) != 0) {
		strcat_s (pszAlarmBuffer, sizeof(pszAlarmBuffer), pszTempBuffer);
		kcounter = 0;
		goto kc2;
	}

	return 0; 
} 
