/* $Workfile: OtxA66CallControl.h $   $Revision: 0 $    $Modtime: 9/28/05 4:29p $  */

/*******************************************************************************

  Copyright (c) Odin TeleSystems Inc., 2006, All Rights Reserved

  @doc PublicDemo

  @module

  OtxA66CallControl.cpp |

  Call control wrapper classes for the Arni boards

*******************************************************************************/
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>

#ifndef WINAPI
#define WINAPI 
#endif

#include "A66HwBasic.h"

#define OtxA66CallControl_API extern

#include "OtxA66CallControl.h"

#include "OsaThrd.h"

#define MAX_DTMF_SEND_STRING 50 // Needed a fixed value, but can be anything

#ifdef __cplusplus
extern "C" {
#endif

/*******************************************************************************
 Global Data
*******************************************************************************/
int g_nLiNumber[DEMO_LI_MAX];
DemoHwEvent g_nOtxA66FxoTaskCallEvent[DEMO_LI_MAX];
OTX_UINT32  g_nOtxA66FxoTaskCallParms[DEMO_LI_MAX]; // Parameter passed by event
OTX_UINT32  g_nOtxA66FxoTaskCallDigitIndex[DEMO_LI_MAX];
char        g_nOtxA66FxoTaskCallDtmfDigit[DEMO_LI_MAX][OTX_A66_CC_MAX_DTMF_DIGITS];
char        *g_pOtxA66FxoTaskCallDigitIndex[DEMO_LI_MAX];
OsaMutexS   g_hOtxA66MutexDtmfDigit;

/*
**
**  ARNI Board Interface Start
**
*/

OTX_RESULT OTX_WINAPI OtxA66BoardInit
  (
    struct DemoEnvT *OtxCallControlSetup, 
    void (*LogUtility)(char *formatStr, va_list argList)
  )
/*******************************************************************************

  @func
  OtxA66OnOpen(DemoEnvT *DemoSetup
               void (*LogUtility)(char *formatStr, va_list argList))

  Used to open the necessary devices and setup interface.

  @comm

*******************************************************************************/
{
  OTX_UINT32 index;
  OTX_RESULT result = OTX_S_OK;  // OTX Standard Return Code

  OsaMutexInit(&g_hOtxA66MutexDtmfDigit);

  // Check for user defined card setup. If Null use default values.
  result = DemoConfigureHardware(OtxCallControlSetup, // User configuration 
                                 NULL);     // Not using the log utility

  if ( OTX_SUCCESS(result) ) {
    result = DemoInitializeDebugConsole();
  }


  if ( OTX_SUCCESS(result) ) {
    result = DemoInitialize(); 
  }

  for(index = 0; index < DEMO_LI_MAX; index++) {
    g_pOtxA66FxoTaskCallDigitIndex[index] = &g_nOtxA66FxoTaskCallDtmfDigit[index][0];
  }

  return(result);
}


OTX_RESULT OTX_WINAPI OtxA66BoardInitEventHandler(
  int potsLiNumber,
  void *hwBase, 
  void (*EventHandler)(void *hwBase, void *hwEvent, void *hwParms, void *potLiNum))
/*******************************************************************************

  @func
  OtxA66BoardInitEventHandler( 
              OTX_UINT32 potsLiNumber,
              void *hwBase, 
              void (*EventHandler)(void *hwBase,  (Value assigned by user)
                                   void *hwEvent, (DemoHwEvent)
                                   void *hwParms, (Value returned by event) 
                                   void *potLiNum (FXO line number)
                                  ))


  This function is used in C application to initialize the callback function.
  Each of the LI's will need to be configured with an event handler, even if
  they are the same object.

  @comm

  This function returns OTX_S_OK if the initialization was successful or
  an OTX error code if initialization failed.

*******************************************************************************/
{
  OTX_RESULT result = OTX_S_OK;  // OTX Standard Return Code

  result = DemoInitializeEventHandler((OTX_UINT32) potsLiNumber,
                                      hwBase, 
                                      EventHandler);
  return(result);

}

OTX_RESULT OTX_WINAPI OtxA66BoardOnClose(void)
/*******************************************************************************

  @func
  OtxA66OnClose()

  Used to close devices that are open and perform cleanup

  @comm

*******************************************************************************/
{
  OTX_RESULT result = OTX_S_OK;  // OTX Standard Return Code

  // Call control function will close any open devices, stop active threads
  // free resources and return card to an inactive state. 
  result = DemoAppExit();

  return(result);
}

int OTX_WINAPI OtxA66BoardGetNumber(int nLiNumber)
/*******************************************************************************

  @func
  OtxA66BoardGetNumber()

  Used to get a board number from a ine number. This function is used by
  the initialization thread to determine which board to assign the line.

  @comm

*******************************************************************************/
{
  int nBoardNumber;

  nBoardNumber =  nLiNumber/DEMO_LI_MAX; 

  return(nBoardNumber);
}


OTX_BOOL OTX_WINAPI OtxA66BoardIsValid(void)
/*******************************************************************************

  @func
  OtxA66PotsLi()

  Used to close devices that are open and perform cleanup

  @comm

*******************************************************************************/
{

  // For now just return OK

  return(OTX_TRUE);
}


/*
**
**  ARNI Board Interface End
**
*/




/*
**
**  FXO Interface Start
**
*/


OTX_BOOL OTX_WINAPI OtxA66FxoIsValid(int nBoardNum, int nLiNumber)
/*******************************************************************************

  @func
  IsValid()

  Returns OTX_TRUE if Li is initialized and working

  @comm

*******************************************************************************/
{
  // For now just verify the line number
  if((nLiNumber >= DEMO_FXO_MAX) || (nLiNumber < 0)) {
    printf("OtxA66FxoIsValid line number not valid %d\n", nLiNumber);

    return(OTX_FALSE);
  }
  else {
    return(OTX_TRUE);
  }

}


OTX_BOOL OTX_WINAPI OtxA66FxoIsOffhook(int nBoardNum, int nLiNumber)
/*******************************************************************************

  @func
  IsValid()

  Returns OTX_TRUE if Li is initialized and working

  @comm

*******************************************************************************/
{
  OTX_BOOL nLiOffHook;
  OTX_RESULT nlastRecordedError;

  if(!OtxA66FxoIsValid(nBoardNum, nLiNumber)){
    // our line number is not correct
    return(1);
  }

  DemoFxoIsLiOffHook(nLiNumber,
                     &nLiOffHook,
                     &nlastRecordedError); 
 
  return(nLiOffHook);
}


int OTX_WINAPI OtxA66FxoGetLiNumber(int nBoardNum, int nLiNumber)
/*******************************************************************************

  @func
  GetLiNumber()

  Return the line number for the li

  @comm

*******************************************************************************/
{
  return(nLiNumber);
}


int OTX_WINAPI OtxA66FxoGoOnhook(int nBoardNum, int nLiNumber)
/*******************************************************************************

  @func
  GoOnhook()

  Hang up the phone. Put into OnHook State

  @comm

*******************************************************************************/
{
  OTX_RESULT nlastRecordedError;
  DemoDeviceCallState nLiState;
  OTX_INT32 nTimeoutCount = 10;
  OTX_BOOL  nLiOffHook    = OTX_TRUE;

  if(!OtxA66FxoIsValid(nBoardNum, nLiNumber)){
    // our line number is not correct
    return(1);
  }

  DemoFxoTerminateCall(nLiNumber, &nLiState);

  while((nLiOffHook == OTX_TRUE) && (nTimeoutCount > 0)) {
    nTimeoutCount--;

    if(nTimeoutCount == 5) {
      // Send call termination again
      DemoFxoTerminateCall(nLiNumber, &nLiState);
    }

    Sleep(100);

    DemoFxoIsLiOffHook(nLiNumber,
                       &nLiOffHook,
                       &nlastRecordedError); 
  }

  return(0);
}


int OTX_WINAPI OtxA66FxoGoOffhook(int nBoardNum, int nLiNumber)
/*******************************************************************************

  @func
  GoOffhook()

  Take the phone off hook and place Li into the OffHook state

  @comm

*******************************************************************************/
{
  OTX_INT32 nTimeoutCount = 10;
  OTX_RESULT nlastRecordedError;
  DemoDeviceCallState nLiState;
  OTX_BOOL  nLiOffHook    = OTX_FALSE;

  if(!OtxA66FxoIsValid(nBoardNum, nLiNumber)){
    // our line number is not correct
    return(1);
  }

  DemoFxoAnswerCall(nLiNumber, &nLiState);

  while((nLiOffHook == OTX_FALSE) && (nTimeoutCount > 0)) {
    nTimeoutCount--;

    if(nTimeoutCount == 5) {
      // Send answer again
      DemoFxoAnswerCall(nLiNumber, &nLiState);
    }

    Sleep(100);

    DemoFxoIsLiOffHook(nLiNumber,
                       &nLiOffHook,
                       &nlastRecordedError); 
  }

  return(0);
}


int OTX_WINAPI OtxA66FxoWaitForCall(int nBoardNum, 
									int nLiNumber, 
                                    char *pszCallerNumberBuffer, 
                                    int cbCallerNumberBuffer)
/*******************************************************************************

  @func
  WaitForCall()

  Wait for a call

  @comm

*******************************************************************************/
{
  OTX_INT32 nRingCnt; // We want the caller ID need to wait for 2nd ring
  OTX_BOOL nLiOffHook;
  OTX_RESULT nlastRecordedError;
  DemoDeviceCallState nLiState;
  char *strCallerId;

  if(!OtxA66FxoIsValid(nBoardNum, nLiNumber)){
    // our line number is not correct
    return(1);
  }

  strCallerId = 0;
  nRingCnt = 0;

  do {
    // There is a new event from the board
    Sleep(100);

    DemoFxoIsLiOffHook(nLiNumber,
                       &nLiOffHook,
                       &nlastRecordedError); 
    if(nLiOffHook == OTX_TRUE) {
      // If Li is already off hook we will never get a call.
      // Put it on hook.
      DemoFxoTerminateCall(nLiNumber,
                           &nLiState);

      Sleep(100);
    }


    switch(g_nOtxA66FxoTaskCallEvent[nLiNumber]){
 
      case HwEvent_unused:
      break;

      case HwEvent_ring:
        // A call is comming in
      case HwEvent_call_answer:
        nRingCnt++;
        if(nRingCnt >= 3) {
          DemoFxoGetCallerId(nLiNumber, 
                             &nLiState,
                             strCallerId);

          if(strCallerId) {
            strncpy(pszCallerNumberBuffer, strCallerId, cbCallerNumberBuffer); 
          }
          else {
            // No Caller ID was available
            *pszCallerNumberBuffer = 0;
          }
        }
        else {
          // Not time to exit 
          g_nOtxA66FxoTaskCallEvent[nLiNumber] = HwEvent_unused;
        }
      break;

      // Not used in the state. Wait for next event.
      case HwEvent_connect:
        g_nOtxA66FxoTaskCallEvent[nLiNumber] = HwEvent_unused;
      break;

      case HwEvent_disconnect:
        g_nOtxA66FxoTaskCallEvent[nLiNumber] = HwEvent_unused;
      break;

      case HwEvent_digit_avail:
        g_nOtxA66FxoTaskCallEvent[nLiNumber] = HwEvent_unused;
      break;

      case HwEvent_init_completed:
        g_nOtxA66FxoTaskCallEvent[nLiNumber] = HwEvent_unused;
      break;

      case HwEvent_buffer_arrived:
        g_nOtxA66FxoTaskCallEvent[nLiNumber] = HwEvent_unused;
      break;

      case HwEvent_ptone_completed:
        g_nOtxA66FxoTaskCallEvent[nLiNumber] = HwEvent_unused;
      break;

      case HwEvent_ring_tone:
        g_nOtxA66FxoTaskCallEvent[nLiNumber] = HwEvent_unused;
      break;

      case HwEvent_busy_tone:
        g_nOtxA66FxoTaskCallEvent[nLiNumber] = HwEvent_unused;
      break;

      case HwEvent_error:
      default:
        // Need to do something with the error
        g_nOtxA66FxoTaskCallEvent[nLiNumber] = HwEvent_unused;
      break;
    }
  } while(g_nOtxA66FxoTaskCallEvent[nLiNumber] == (int) HwEvent_unused);

  // clear the event that allowed us to exit the wait
  g_nOtxA66FxoTaskCallEvent[nLiNumber] = HwEvent_unused;

  return(0);
}

int OTX_WINAPI OtxA66FxoWaitForDtmf(int nBoardNum, 
									int nLiNumber, 
                                    int nWaitTimeInMilliseconds, 
                                    char *pszDtmfBuffer, 
                                    int cbDtmfBuffer)
/*******************************************************************************

  @func
  WaitForDtmf()

  Wait for a string of DTMF digits

  @comm

*******************************************************************************/
{
  OTX_INT32 nDigitCnt   = cbDtmfBuffer;
  OTX_INT32 nWaitCnt    = nWaitTimeInMilliseconds;
  OTX_INT32 nSleepTime  = 20;
  OTX_INT32 nDigCnt;
  OTX_BOOL nLiOffHook;
  OTX_BOOL ndigitStart = OTX_FALSE;
  OTX_RESULT nlastRecordedError;
  DemoDeviceCallState nLiState;
  char *strDtmfDig;


  if(!OtxA66FxoIsValid(nBoardNum, nLiNumber)){
    // our line number is not correct
    return(1);
  }

  strDtmfDig = pszDtmfBuffer;
  *strDtmfDig = '\0';

  OsaMutexLock(&g_hOtxA66MutexDtmfDigit);
  g_nOtxA66FxoTaskCallDigitIndex[nLiNumber] = 0; // Start with empty input buffer
  OsaMutexUnlock(&g_hOtxA66MutexDtmfDigit);

  do {
    // There is a new event from the board
    Sleep(nSleepTime);

    // The task was asleep for nSleepTime make sure we do not loop forever
    nWaitCnt = nWaitCnt - nSleepTime;

    DemoFxoIsLiOffHook(nLiNumber,
                       &nLiOffHook,
                       &nlastRecordedError); 
    if(nLiOffHook == OTX_FALSE) {
      // If Li should be offhook. Code is out of sync.
      nDigitCnt = 0; // Terminate the wait

      DemoFxoTerminateCall(nLiNumber,
                           &nLiState);

      Sleep(nSleepTime);
    }


    switch(g_nOtxA66FxoTaskCallEvent[nLiNumber]){
 
      case HwEvent_unused:
      break;

      case HwEvent_ring:
        g_nOtxA66FxoTaskCallEvent[nLiNumber] = HwEvent_unused;
      break;

      case HwEvent_call_answer:
        g_nOtxA66FxoTaskCallEvent[nLiNumber] = HwEvent_unused;
      break;

      // Not used in the state. Wait for next event.
      case HwEvent_connect:
        g_nOtxA66FxoTaskCallEvent[nLiNumber] = HwEvent_unused;
      break;

      case HwEvent_disconnect:
        g_nOtxA66FxoTaskCallEvent[nLiNumber] = HwEvent_unused;
      break;

      case HwEvent_digit_avail:

        // This is what we were waiting on
        nDigCnt = 0;
        OsaMutexLock(&g_hOtxA66MutexDtmfDigit);
        while(nDigCnt < (OTX_INT32) g_nOtxA66FxoTaskCallDigitIndex[nLiNumber]) {
          *strDtmfDig = g_nOtxA66FxoTaskCallDtmfDigit[nLiNumber][nDigCnt];
          strDtmfDig++; // Next address
          *strDtmfDig = '\0'; // end with a null
          nDigCnt = nDigCnt + 2;
          nDigitCnt--;
          if(nDigitCnt == 0) {
            // Finished collecting digits exit 
            nDigCnt = g_nOtxA66FxoTaskCallDigitIndex[nLiNumber];
          }
        }
        g_nOtxA66FxoTaskCallDigitIndex[nLiNumber] = 0;
        OsaMutexUnlock(&g_hOtxA66MutexDtmfDigit);

        printf("OtxA66FxoWaitForDtmf Li(%d) -%s-\n", nLiNumber, pszDtmfBuffer);

        ndigitStart = OTX_TRUE; // DTMF digit
        nWaitCnt    = nWaitTimeInMilliseconds;
        g_nOtxA66FxoTaskCallEvent[nLiNumber] = HwEvent_unused;

      break;

      case HwEvent_init_completed:
        g_nOtxA66FxoTaskCallEvent[nLiNumber] = HwEvent_unused;
      break;

      case HwEvent_buffer_arrived:
        g_nOtxA66FxoTaskCallEvent[nLiNumber] = HwEvent_unused;
      break;

      case HwEvent_ptone_completed:
        g_nOtxA66FxoTaskCallEvent[nLiNumber] = HwEvent_unused;
      break;

      case HwEvent_ring_tone:
        g_nOtxA66FxoTaskCallEvent[nLiNumber] = HwEvent_unused;
      break;

      case HwEvent_busy_tone:
        g_nOtxA66FxoTaskCallEvent[nLiNumber] = HwEvent_unused;
      break;

      case HwEvent_error:
      default:
        // Need to do something with the error
        g_nOtxA66FxoTaskCallEvent[nLiNumber] = HwEvent_unused;
     break;
    }

  } while((nDigitCnt > 0) && (nWaitCnt  > 0));

  // Make sure a NULL is at the end of the string
  *strDtmfDig = '\0';

  // clear the event that allowed us to exit the wait
  g_nOtxA66FxoTaskCallEvent[nLiNumber] = HwEvent_unused;

  printf("OtxA66FxoWaitForDtmf Li(%d) Exit function -%s-\n", nLiNumber, pszDtmfBuffer);

  return(0);
}

int OTX_WINAPI OtxA66FxoSendDtmf(int nBoardNum, int nLiNumber, char *pszDtmf)
/*******************************************************************************

  @func
  SendDtmf()

  Send a string of DTMF digits
  When DTMF digits are sent they are also detected by the DTFM detector.
  If echo cancel device is being used the DTMF will not be received.

  @comm

*******************************************************************************/
{
  int nStrSize;
  char pszDtmfBuffer[MAX_DTMF_SEND_STRING];
  
  DemoDeviceCallState nLiState;

  if(!OtxA66FxoIsValid(nBoardNum, nLiNumber)){
    // our line number is not correct
    return(1);
  }

  nStrSize = strlen(pszDtmf);
  if(nStrSize > MAX_DTMF_SEND_STRING - 1) {
    printf("OtxA66FxoSendDtmf DTMF string larger then buffer %d\n", nStrSize);
    nStrSize = MAX_DTMF_SEND_STRING - 1;
  }

  memset(pszDtmfBuffer, '\0', MAX_DTMF_SEND_STRING);
  
  DemoFxoSendDtmfDigits(nLiNumber,
                        &nLiState,  
                        pszDtmf);

  OtxA66FxoWaitForDtmf(nBoardNum, 
	                   nLiNumber, 
                       850 * nStrSize, // longer strings will take more time.
                       pszDtmfBuffer, 
                       nStrSize);
  return(0);
}

/*
**
**  FXO Interface End
**
*/

#ifdef __cplusplus
}
#endif

/*--------------------------------------------------------------------------*\

  Copyright (C) Odin TeleSystems Inc., 2006


  All rights reserved. No part of this computer program may be reproduced,
  stored in any retrieval system, or transmitted, in any form or by any
  means, electronic, mechanical, photocopying, recording, or otherwise,
  without the prior written permission of Odin TeleSystems Inc., or without
  a permission stipulated in the agreement/contract under which this computer
  program was supplied.

\*--------------------------------------------------------------------------*/
