/* $Workfile: OdDevice.h $   $Revision: 68 $    $Modtime: 9/28/05 4:29p $  */
#ifndef OTX_A66_CALL_CONTROL_H
#define OTX_A66_CALL_CONTROL_H
/*******************************************************************************

  Copyright (c) Odin TeleSystems Inc., 2006, All Rights Reserved

  @doc PublicDemo

  @module

  OtxA66CallControl.h |

  Call control wrapper classes for the Arni boards

*******************************************************************************/
#ifdef __cplusplus
extern "C" {
#endif

#ifdef __cplusplus
#define OtxA66CallControl_API extern "C"
#else
#define OtxA66CallControl_API extern
#endif

#ifndef OtxA66CallControl_API
#define OtxA66CallControl_API extern
#endif

#ifndef OTX_WINAPI
#define OTX_WINAPI 
#endif

#include <stdio.h>
#include <stdarg.h>
#include <OtxErr.h>
#include <OtxDrv.h>
#include <OtxDataD.h>
#include <OtxCstD.h>


#define IN
#define OUT   


#define OTX_A66_CC_MAX_DTMF_DIGITS 50

/*******************************************************************************

  Represents line interface on the Arni board.

*******************************************************************************/
  // @cmember returns true if the device was constructed without errors
  OtxA66CallControl_API OTX_BOOL OTX_WINAPI OtxA66FxoIsValid(int nBoardNum, int nLiNumber);

  // @cmember Return the Li number of this Li
  OtxA66CallControl_API int OTX_WINAPI OtxA66FxoGetLiNumber(int nLiNumber);

  // @cmember Take the device offhook (answer)
  OtxA66CallControl_API OTX_BOOL OTX_WINAPI OtxA66FxoIsOffhook(int nBoardNum, int nLiNumber);

  // @cmember Take the device offhook (answer)
  OtxA66CallControl_API int OTX_WINAPI OtxA66FxoGoOffhook(int nBoardNum, int nLiNumber);

  // @cmember Hang up 
  OtxA66CallControl_API int OTX_WINAPI OtxA66FxoGoOnhook(int nBoardNum, int nLiNumber);

  // @cmember Wait for call. Return Caller number when call occurs
  OtxA66CallControl_API int OTX_WINAPI OtxA66FxoWaitForCall(int nBoardNum, int nLiNumber, char *pszCallerNumberBuffer, int cbCallerNumberBuffer);

  // @cmember Wait DTMF digits for specified time. Return received DTMF digits.
  OtxA66CallControl_API int OTX_WINAPI OtxA66FxoWaitForDtmf(int nBoardNum, int nLiNumber, int nWaitTimeInMilliseconds, char *pszDtmfBuffer, int cbDtmfBuffer);

  // @cmember Send a string of DTMF digits
  OtxA66CallControl_API int OTX_WINAPI OtxA66FxoSendDtmf(int nBoardNum, int nLiNumber, char *pszDtmf);


/*******************************************************************************
  @class


  Represents Arni board interface.

*******************************************************************************/
  OtxA66CallControl_API OTX_RESULT OTX_WINAPI OtxA66BoardInit
    (    
      struct DemoEnvT *OtxCallControlSetup, // User defines card initialization
      void (*LogUtility)(char *formatStr, va_list argList) // User log callback
    );

  OtxA66CallControl_API int OTX_WINAPI OtxA66BoardGetNumber(int nLiNumber);

  OtxA66CallControl_API OTX_RESULT OTX_WINAPI OtxA66BoardInitEventHandler
    (
      int potsLiNumber,
      void *hwBase, 
      void (*EventHandler)(void *, void *, void *, void *)
    );

  OtxA66CallControl_API OTX_RESULT OTX_WINAPI OtxA66BoardOnClose(void);

  // @cmember returns true if the device was constructed without errors
  OtxA66CallControl_API OTX_BOOL OTX_WINAPI OtxA66BoardIsValid(void);



/*--------------------------------------------------------------------------*\

  Copyright (C) Odin TeleSystems Inc., 2006


  All rights reserved. No part of this computer program may be reproduced,
  stored in any retrieval system, or transmitted, in any form or by any
  means, electronic, mechanical, photocopying, recording, or otherwise,
  without the prior written permission of Odin TeleSystems Inc., or without
  a permission stipulated in the agreement/contract under which this computer
  program was supplied.

\*--------------------------------------------------------------------------*/

#ifdef __cplusplus
}
#endif

#endif   // OTX_A66_CALL_CONTROL_H

